@@ -4,10 +4,11 @@ import pandas as pd
 from binance.client import Client
 
 from OpenBacktest.ObtUtility import Colors, timeframes, divide
-from OpenBacktest.ObtWallet import SymmetricWallet, AsymmetricWallet
-
 
 # Define a symmetric backtest engine
+from OpenBacktest.ObtWallet import Wallet
+
+
 class Engine:
     # Initialising the class
     def __init__(self, container, output=True):
@@ -22,8 +23,7 @@ class Engine:
         self.client = Client()
 
         # Used to run a backtest
-        self.buy_condition = None
-        self.sell_condition = None
+        self.strategy = None
         self.wallet = None
 
         # TP & SL
@@ -42,53 +42,47 @@ class Engine:
         return self.container.get_pair(name).dataframe
 
     # -------------------------------------------------------------------------------
-    # def backtest parameters to run a simple strategy with buy & sell condition
-    def register_sell_and_buy_condition(self, buy_condition, sell_condition):
-        self.buy_condition = buy_condition
-        self.sell_condition = sell_condition
+    # def backtest parameters to run an asymmetric strategy
+    def register_strategy(self, strategy):
+        self.strategy = strategy
 
-    # run a simple backtest with buy & sell condition
-    def run_sell_and_buy_condition(self, coin_name, token_name, coin_balance, token_balance, taker, maker, finish=True):
+    # run an asymmetric strategy
+    def run_strategy(self, coin_name, token_name, coin_balance, token_balance, taker, finish=True):
         # condition not None test
-        if self.buy_condition is None or self.sell_condition is None:
-            print(Colors.RED + "Error, you can't run a backtest because you don't have buy & sell condition functions "
-                               "registered")
+        if self.strategy is None:
+            print(Colors.RED + "Error, you can't run a backtest because you don't have a strategy function registered")
             return
 
         # Wallet initialisation
-        self.wallet = SymmetricWallet(coin_name, token_name, coin_balance, token_balance, taker, maker,
-                                      self.main_dataframe())
+        self.wallet = Wallet(coin_name, token_name, coin_balance, token_balance, taker, self.main_dataframe())
         # Ini
         index = 0
+        percent = 0
 
         # Main loop
         while index <= self.container.main.max_index:
 
+            p = divide(100 * index, self.container.main.max_index)
+            if p > percent + 5:
+                print(Colors.YELLOW + "Progress: " + str(round(p)) + "%")
+                percent = p
+
+            # main
+            report = self.strategy(self.main_dataframe(), index)
+            if report is not None:
+                if report.order == "buy":
+                    self.wallet.buy(index, report.amount, report.percent_amount)
+                elif report.order == "sell":
+                    self.wallet.sell(index, report.amount, report.percent_amount)
+
             # update tp & sl
             self.update_stop(index)
 
-            if self.buy_condition(self.main_dataframe(), index) and self.wallet.coin_balance > 0:
-                if self.wallet.position_book.last_position is None or self.wallet.position_book.last_position.closed:
-                    self.wallet.buy(index)
-                else:
-                    print(Colors.YELLOW, "Advert, buy order is not placeable because a long position is already opened")
-            elif self.sell_condition(self.main_dataframe(), index) and self.wallet.token_balance > 0:
-                if self.wallet.position_book.last_position is not None and not \
-                        self.wallet.position_book.last_position.closed:
-                    self.wallet.sell(index)
-                else:
-                    print(Colors.YELLOW, "Advert, sell order is not placeable because there's no opened long position")
-
-            # update balance
-            self.balance.append(self.wallet.get_current_wallet_value(index))
-
             # end
             index += 1
 
-        self.main_dataframe()["balance"] = self.balance
-
         # Sell all remaining coins
-        if self.wallet.token_balance > 0 and finish:
+        if self.wallet.wallet_frame.iloc[-1]["token_balance"] > 0 and finish:
             self.wallet.sell(self.container.main.max_index)
 
     # -------------------------------------------------------------------------------
@@ -158,10 +152,6 @@ class AsymmetricEngine:
         self.strategy = None
         self.wallet = None
 
-        # TP & SL
-        self.tp = None
-        self.sl = None
-
         # balance ( used for balance graph plotting )
         self.balance = []
 
@@ -202,9 +192,6 @@ class AsymmetricEngine:
                 elif report.order == "sell":
                     self.wallet.sell(index, report.amount, report.percent_amount)
 
-            # update tp & sl
-            self.update_stop(index)
-
             # update balance
             self.balance.append(self.wallet.get_current_wallet_value(index))
 
@@ -223,48 +210,6 @@ class AsymmetricEngine:
     def get_sub_dataframe(self, name):
         return self.container.get_pair(name).dataframe
 
-    # set TP & SL
-    def set_take_profit(self, index, target=None, percent_target=None):
-        if target is None and percent_target is None:
-            print(Colors.LIGHT_RED, "Error we can't register your take profit, no target")
-
-        if target is None:
-            if percent_target is None:
-                print(Colors.LIGHT_RED, "Error we can't register your take profit, no target")
-                exit()
-            else:
-                target = self.main_dataframe()["close"][index] + \
-                         self.main_dataframe()["close"][index] * divide(percent_target, 100)
-        self.tp = target
-
-    def set_stop_loss(self, index, target=None, percent_target=None):
-        if target is None and percent_target is None:
-            print(Colors.LIGHT_RED, "Error we can't register your stop loss, no target")
-
-        if target is None:
-            if percent_target is None:
-                print(Colors.LIGHT_RED, "Error we can't register your stop loss, no target")
-                exit()
-            else:
-                target = self.main_dataframe()["close"][index] + \
-                         self.main_dataframe()["close"][index] * divide(percent_target, 100)
-        self.sl = target
-
-    def cancel_take_profit(self):
-        self.tp = None
-
-    def cancel_stop_loss(self):
-        self.sl = None
-
-    def update_stop(self, index):
-        price = self.main_dataframe()["close"][index]
-        if self.tp is not None and self.tp <= price:
-            self.wallet.sell(index)
-            self.tp = None
-        if self.sl is not None and self.sl >= price:
-            self.wallet.sell(index)
-            self.sl = None
-
 
 # define a pair
 class Pair:
@@ -359,7 +304,7 @@ class Pair:
             index = self.timestamps_with_index[closest_value] - 1
 
         if index < 0:
-            print(Colors.RED + "Error ! The row you are trying to get the equivalent index of an alt dataframe is "
+            print(Colors.RED + "Error ! The row you're trying to get the equivalent index is "
                                "older than the beginning of this dataframe")
             exit()
 
